@@ -12,7 +12,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <openais/saAis.h>
-#include <openais/confdb.h>
+#include <corosync/confdb.h>
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
 
@@ -20,7 +20,6 @@
 
 /* Callbacks are not supported - we will use them to update fullxml doc/ctx */
 static confdb_callbacks_t callbacks = {
-	.confdb_change_notify_fn = NULL,
 };
 
 static confdb_handle_t handle = 0;
