@@ -23,7 +23,9 @@
 #include "simple_auth.h"
 #include "mcast.h"
 #include "options.h"
+#include "debug.h"
 
+LOGSYS_DECLARE_SUBSYS("XVM", SYSLOGLEVEL);
 
 
 /* Assignment functions */
@@ -64,7 +66,7 @@ assign_family(fence_xvm_args_t *args, struct arg_info *arg,
 	} else if (!strcasecmp(value, "auto")) {
 		args->family = 0;
 	} else {
-		printf("Unsupported family: '%s'\n", value);
+		log_printf(LOG_ERR, "Unsupported family: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -95,25 +97,18 @@ assign_port(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	args->port = atoi(value);
 	if (args->port <= 0 || args->port >= 65500) {
-		printf("Invalid port: '%s'\n", value);
+		log_printf(LOG_ERR, "Invalid port: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
 
 
-static inline void
-assign_interface(fence_xvm_args_t *args, struct arg_info *arg, char *value)
-{
-	args->ifindex = if_nametoindex(value);
-}
-
-
 static inline void
 assign_retrans(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	args->retr_time = atoi(value);
 	if (args->retr_time <= 0) {
-		printf("Invalid retransmit time: '%s'\n", value);
+		log_printf(LOG_ERR, "Invalid retransmit time: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -130,7 +125,7 @@ assign_hash(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	} else if (!strcasecmp(value, "sha512")) {
 		args->hash = HASH_SHA512;
 	} else {
-		printf("Unsupported hash: %s\n", value);
+		log_printf(LOG_ERR, "Unsupported hash: %s\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -148,7 +143,7 @@ assign_auth(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	} else if (!strcasecmp(value, "sha512")) {
 		args->auth = AUTH_SHA512;
 	} else {
-		printf("Unsupported auth type: %s\n", value);
+		log_printf(LOG_ERR, "Unsupported auth type: %s\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -163,8 +158,8 @@ assign_key(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	args->key_file = strdup(value);
 
 	if (stat(value, &st) == -1) {
-		printf("Invalid key file: '%s' (%s)\n", value,
-		       strerror(errno));
+ 		log_printf(LOG_ERR, "Invalid key file: '%s' (%s)\n", value,
+  		       strerror(errno));
 		args->flags |= F_ERR;
 	}
 }
@@ -180,7 +175,7 @@ assign_op(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 	} else if (!strcasecmp(value, "reboot")) {
 		args->op = FENCE_REBOOT;
 	} else {
-		printf("Unsupported operation: %s\n", value);
+		log_printf(LOG_ERR, "Unsupported operation: %s\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -190,21 +185,24 @@ static inline void
 assign_domain(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	if (args->domain) {
-		printf("Domain/UUID may not be specified more than once\n");
+		log_printf(LOG_ERR,
+		   "Domain/UUID may not be specified more than once\n");
 		args->flags |= F_ERR;
 		return;
 	}
 
+	if (args->domain)
+		free(args->domain);
 	args->domain = strdup(value);
 
 	if (strlen(value) <= 0) {
-		printf("Invalid domain name\n");
+		log_printf(LOG_ERR, "Invalid domain name\n");
 		args->flags |= F_ERR;
 	}
 
 	if (strlen(value) >= MAX_DOMAINNAME_LENGTH) {
 		errno = ENAMETOOLONG;
-		printf("Invalid domain name: '%s' (%s)\n",
+		log_printf(LOG_ERR, "Invalid domain name: '%s' (%s)\n",
 		       value, strerror(errno));
 		args->flags |= F_ERR;
 	}
@@ -230,7 +228,7 @@ assign_timeout(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 {
 	args->timeout = atoi(value);
 	if (args->timeout <= 0) {
-		printf("Invalid timeout: '%s'\n", value);
+		log_printf(LOG_ERR, "Invalid timeout: '%s'\n", value);
 		args->flags |= F_ERR;
 	}
 }
@@ -275,7 +273,6 @@ assign_uri(fence_xvm_args_t *args, struct arg_info *arg, char *value)
 }
 
 
-
 /** ALL valid command line and stdin arguments for this fencing agent */
 static struct arg_info _arg_info[] = {
 	{ '\xff', NULL, "agent",
@@ -310,10 +307,6 @@ static struct arg_info _arg_info[] = {
 	  "IP port (default=1229)",
 	  assign_port },
 
-	{ 'I', "-I <interface>", "multicast_address",
-	  "Network interface name to listen on",
-	  assign_interface },
-
 	{ 'r', "-r <retrans>", "retrans", 
 	  "Multicast retransmit time (in 1/10sec; default=20)",
 	  assign_retrans },
@@ -423,7 +416,6 @@ args_init(fence_xvm_args_t *args)
 	args->hash = DEFAULT_HASH;
 	args->auth = DEFAULT_AUTH;
 	args->port = 1229;
-	args->ifindex = 0;
 	args->family = PF_INET;
 	args->timeout = 30;
 	args->retr_time = 20;
@@ -433,8 +425,8 @@ args_init(fence_xvm_args_t *args)
 }
 
 
-#define _pr_int(piece) printf("  %s = %d\n", #piece, piece)
-#define _pr_str(piece) printf("  %s = %s\n", #piece, piece)
+#define _pr_int(piece) dbg_printf(1, "  %s = %d\n", #piece, piece)
+#define _pr_str(piece) dbg_printf(1, "  %s = %s\n", #piece, piece)
 
 
 /**
@@ -445,7 +437,7 @@ args_init(fence_xvm_args_t *args)
 void
 args_print(fence_xvm_args_t *args)
 {
-	printf("-- args @ %p --\n", args);
+	dbg_printf(1, "-- args @ %p --\n", args);
 	_pr_str(args->addr);
 	_pr_str(args->domain);
 	_pr_str(args->key_file);
@@ -453,13 +445,12 @@ args_print(fence_xvm_args_t *args)
 	_pr_int(args->hash);
 	_pr_int(args->auth);
 	_pr_int(args->port);
-	_pr_int(args->ifindex);
 	_pr_int(args->family);
 	_pr_int(args->timeout);
 	_pr_int(args->retr_time);
 	_pr_int(args->flags);
 	_pr_int(args->debug);
-	printf("-- end args --\n");
+	dbg_printf(1, "-- end args --\n");
 }
 
 
@@ -467,6 +458,8 @@ args_print(fence_xvm_args_t *args)
   Print out arguments and help information based on what is allowed in
   the getopt string optstr.
 
+  TODO use log_printf instead of printf 
+
   @param progname	Program name.
   @param optstr		Getopt(3) style options string
   @param print_stdin	0 = print command line options + description,
