@@ -5,9 +5,21 @@
 #define BUFLEN		MAX_NODENAME_LEN+1
 
 static cman_handle_t	ch;
+static cman_handle_t	ch_admin;
 static cman_node_t	cman_nodes[MAX_NODES];
 static int		cman_node_count;
 
+void kick_node_from_cluster(int nodeid)
+{
+	if (!nodeid) {
+		log_error("telling cman to shut down cluster locally");
+		cman_shutdown(ch_admin, CMAN_SHUTDOWN_ANYWAY);
+	} else {
+		log_error("telling cman to remove nodeid %d from cluster",
+			  nodeid);
+		cman_kill_node(ch_admin, nodeid);
+	}
+}
 
 static int name_equal(char *name1, char *name2)
 {
@@ -146,12 +158,18 @@ int setup_cman(void)
 	int init = 0, active = 0;
 
  retry_init:
-	ch = cman_init(NULL);
-	if (!ch) {
+	ch_admin = cman_admin_init(NULL);
+	if (!ch_admin) {
 		if (init++ < 2) {
 			sleep(1);
 			goto retry_init;
 		}
+		log_error("cman_admin_init error %d", errno);
+		return -ENOTCONN;
+	}
+
+	ch = cman_init(NULL);
+	if (!ch) {
 		log_error("cman_init error %d", errno);
 		return -ENOTCONN;
 	}
