@@ -229,30 +229,31 @@ void are_you_sure(commandline_t *comline)
 
 
 /**
- * check_mount -
+ * check_mount - check to see if device is mounted/busy
  * @
  *
  */
 
 void check_mount(char *device)
 {
-	struct mntent *mnt;
-	FILE *fp;
+	struct stat st_buf;
+	int fd;
 
-	if ((fp = setmntent("/proc/mounts", "r")) == NULL) {
-		die("error opening /proc/mounts");
-	}
+	if (stat(device, &st_buf) < 0)
+		die("could not stat device %s\n", device);
+	if (!S_ISBLK(st_buf.st_mode))
+		die("%s is not a block device\n", device);
 
-	while ((mnt = getmntent(fp)) != NULL) {
-		if (strcmp(device, mnt->mnt_fsname) == 0) {
-			printf("cannot create filesystem: ");
-			printf("%s appears to be mounted\n", device);
-			endmntent(fp);
-			exit(EXIT_FAILURE);
+	fd = open(device, O_RDONLY | O_NONBLOCK | O_EXCL);
+
+	if (fd < 0) {
+		if (errno == EBUSY) {
+			die("device %s is busy\n", device);
 		}
 	}
-
-	endmntent(fp);
+	else {
+		close(fd);
+	}
 
 	return;
 }
