@@ -92,6 +92,37 @@ static int get_mountgroup_name(uint32_t mg_id)
 	return rv;
 }
 
+/* This is for the case where dlm_controld exits/fails, abandoning dlm
+   lockspaces in the kernel, and then dlm_controld is restarted.  When
+   dlm_controld exits and abandons lockspaces, that node needs to be
+   rebooted to clear the uncontrolled lockspaces from the kernel. */
+
+int check_uncontrolled_lockspaces(void)
+{
+	DIR *d;
+	struct dirent *de;
+	int count = 0;
+
+	d = opendir(DLM_SYSFS_DIR);
+	if (!d)
+		return 0;
+
+	while ((de = readdir(d))) {
+		if (de->d_name[0] == '.')
+			continue;
+
+		log_error("found uncontrolled lockspace %s", de->d_name);
+		count++;
+	}
+	closedir(d);
+
+	if (count) {
+		kick_node_from_cluster(our_nodeid);
+		return -1;
+	}
+	return 0;
+}
+
 /* find the mountgroup with "mg_id" in sysfs, get it's name, then look for
    the ls with with the same name in lockspaces list, return its id */
 
@@ -323,8 +354,8 @@ int set_configfs_members(char *name, int new_count, int *new_members,
 		else if (id_exists(id, old_count, old_members))
 			continue;
 
-		if (!is_cman_member(id))
-			cman_statechange();
+		if (!is_cluster_member(id))
+			update_cluster();
 		/*
 		 * create node's dir
 		 */
@@ -782,7 +813,7 @@ int setup_configfs(void)
 		return rv;
 
 	/* add configfs entries for existing nodes */
-	cman_statechange();
+	update_cluster();
 
 	/* the kernel has its own defaults for these values which we
 	   don't want to change unless these have been set; -1 means
@@ -798,3 +829,103 @@ int setup_configfs(void)
 	return 0;
 }
 
+static void find_minors(void)
+{
+	FILE *fl;
+	char name[256];
+	uint32_t number;
+	int found = 0;
+	int c;
+
+	if (!(fl = fopen("/proc/misc", "r"))) {
+		log_error("/proc/misc fopen failed: %s", strerror(errno));
+		return;
+	}
+
+	while (!feof(fl)) {
+		if (fscanf(fl, "%d %255s\n", &number, &name[0]) == 2) {
+
+			if (!strcmp(name, "dlm-control")) {
+				control_minor = number;
+				found++;
+			} else if (!strcmp(name, "dlm-monitor")) {
+				monitor_minor = number;
+				found++;
+			} else if (!strcmp(name, "dlm_plock")) {
+				plock_minor = number;
+				found++;
+			} else if (!strcmp(name, "lock_dlm_plock")) {
+				old_plock_minor = number;
+				found++;
+			}
+
+		} else do {
+			c = fgetc(fl);
+		} while (c != EOF && c != '\n');
+
+		if (found == 3)
+			break;
+	}
+	fclose(fl);
+
+	if (!found)
+		log_error("Is dlm missing from kernel? No misc devices found.");
+}
+
+static int find_udev_device(char *path, uint32_t minor)
+{
+	struct stat st;
+	int i;
+
+	for (i = 0; i < 10; i++) {
+		if (stat(path, &st) == 0 && minor(st.st_rdev) == minor)
+			return 0;
+		sleep(1);
+	}
+
+	log_error("cannot find device %s with minor %d", path, minor);
+	return -1;
+}
+
+int setup_misc_devices(void)
+{
+	int rv;
+
+	find_minors();
+
+	if (control_minor) {
+		rv = find_udev_device("/dev/misc/dlm-control", control_minor);
+		if (rv < 0)
+			return rv;
+		log_debug("found /dev/misc/dlm-control minor %u",
+			  control_minor);
+	}
+
+	if (monitor_minor) {
+		rv = find_udev_device("/dev/misc/dlm-monitor", monitor_minor);
+		if (rv < 0)
+			return rv;
+		log_debug("found /dev/misc/dlm-monitor minor %u",
+			  monitor_minor);
+	}
+
+	if (plock_minor) {
+		rv = find_udev_device("/dev/misc/dlm_plock", plock_minor);
+		if (rv < 0)
+			return rv;
+		log_debug("found /dev/misc/dlm_plock minor %u",
+			  plock_minor);
+	}
+
+	if (old_plock_minor) {
+		rv = find_udev_device("/dev/misc/lock_dlm_plock",
+				      old_plock_minor);
+		if (rv < 0)
+			return rv;
+		log_debug("found /dev/misc/lock_dlm_plock minor %u",
+			  old_plock_minor);
+	}
+
+	return 0;
+}
+
