@@ -32,8 +32,8 @@
 #include <dirent.h>
 #include <openais/saAis.h>
 #include <openais/saCkpt.h>
-#include <openais/cpg.h>
-#include <openais/service/logsys.h>
+#include <corosync/cpg.h>
+#include <corosync/engine/logsys.h>
 
 #include <linux/dlmconstants.h>
 #include "libdlmcontrol.h"
@@ -46,13 +46,13 @@
    The libcpg limit is larger at CPG_MAX_NAME_LENGTH 128.  Our cpg name includes
    a "dlm:" prefix before the lockspace name. */
 
-/* Maximum members of a lockspace, should match CPG_MEMBERS_MAX in openais/cpg.h.
+/* Maximum members of a lockspace, should match CPG_MEMBERS_MAX in corosync/cpg.h.
    There are no max defines in dlm-kernel for lockspace members. */
 
 #define MAX_NODES	128
 
 /* Maximum number of IP addresses per node, when using SCTP and multi-ring in
-   openais.  In dlm-kernel this is DLM_MAX_ADDR_COUNT, currently 3. */
+   corosync  In dlm-kernel this is DLM_MAX_ADDR_COUNT, currently 3. */
 
 #define MAX_NODE_ADDRESSES 4
 
@@ -74,7 +74,7 @@ extern int poll_ignore_plock;
 extern int plock_fd;
 extern int plock_ci;
 extern struct list_head lockspaces;
-extern int cman_quorate;
+extern int cluster_quorate;
 extern int our_nodeid;
 extern char daemon_debug_buf[256];
 extern char dump_buf[DLMC_DUMP_SIZE];
@@ -83,6 +83,10 @@ extern int dump_wrap;
 extern char plock_dump_buf[DLMC_DUMP_SIZE];
 extern int plock_dump_len;
 extern int group_mode;
+extern uint32_t control_minor;
+extern uint32_t monitor_minor;
+extern uint32_t plock_minor;
+extern uint32_t old_plock_minor;
 
 void daemon_dump_save(void);
 
@@ -217,6 +221,8 @@ int add_configfs_node(int nodeid, char *addr, int addrlen, int local);
 void del_configfs_node(int nodeid);
 void clear_configfs(void);
 int setup_configfs(void);
+int check_uncontrolled_lockspaces(void);
+int setup_misc_devices(void);
 
 /* config.c */
 int setup_ccs(void);
@@ -264,12 +270,15 @@ char *dlm_mode_str(int mode);
 void cluster_dead(int ci);
 
 /* member_cman.c */
-int setup_cman(void);
-void close_cman(void);
-void process_cman(int ci);
-void cman_statechange(void);
-int is_cman_member(int nodeid);
+int setup_cluster(void);
+void close_cluster(void);
+void process_cluster(int ci);
+void update_cluster(void);
+int is_cluster_member(int nodeid);
 char *nodeid2name(int nodeid);
+void kick_node_from_cluster(int nodeid);
+int fence_node_time(int nodeid, uint64_t *last_fenced_time);
+int fence_in_progress(int *count);
 
 /* netlink.c */
 int setup_netlink(void);
@@ -291,19 +300,38 @@ void purge_plocks(struct lockspace *ls, int nodeid, int unmount);
 int fill_plock_dump_buf(struct lockspace *ls);
 
 /* group.c */
+#define BUILD_GROUPD_COMPAT
+#ifdef BUILD_GROUPD_COMPAT
 int setup_groupd(void);
 void close_groupd(void);
 void process_groupd(int ci);
 int dlm_join_lockspace_group(struct lockspace *ls);
 int dlm_leave_lockspace_group(struct lockspace *ls);
 int set_node_info_group(struct lockspace *ls, int nodeid,
-			struct dlmc_node *node);
+	struct dlmc_node *node);
 int set_lockspace_info_group(struct lockspace *ls,
-			struct dlmc_lockspace *lockspace);
-int set_lockspaces_group(int *count, struct dlmc_lockspace **lss_out);
-int set_lockspace_nodes_group(struct lockspace *ls, int option, int *node_count,
-			struct dlmc_node **nodes);
+	struct dlmc_lockspace *lockspace);
+int set_lockspaces_group(int *count,
+	struct dlmc_lockspace **lss_out);
+int set_lockspace_nodes_group(struct lockspace *ls, int option,
+	int *node_count, struct dlmc_node **nodes);
 void set_group_mode(void);
+#else
+static inline int setup_groupd(void) { return -1; }
+static inline void close_groupd(void) { }
+static inline void process_groupd(int ci) { }
+static inline int dlm_join_lockspace_group(struct lockspace *ls) { return -1; }
+static inline int dlm_leave_lockspace_group(struct lockspace *ls) { return -1; }
+static inline int set_node_info_group(struct lockspace *ls, int nodeid,
+	struct dlmc_node *node) { return -1; }
+static inline int set_lockspace_info_group(struct lockspace *ls,
+	struct dlmc_lockspace *lockspace) { return -1; }
+static inline int set_lockspaces_group(int *count,
+	struct dlmc_lockspace **lss_out) { return -1; }
+static inline int set_lockspace_nodes_group(struct lockspace *ls, int option,
+	int *node_count, struct dlmc_node **nodes) { return -1; }
+static inline void set_group_mode(void) { }
+#endif
 
 /* logging.c */
 
