@@ -18,6 +18,7 @@ static struct pollfd *pollfd = NULL;
 static pthread_t query_thread;
 static pthread_mutex_t query_mutex;
 static struct list_head fs_register_list;
+static int kernel_monitor_fd;
 
 struct client {
 	int fd;
@@ -198,9 +199,23 @@ struct fs_reg {
 	char name[DLM_LOCKSPACE_LEN+1];
 };
 
+static int fs_register_check(char *name)
+{
+	struct fs_reg *fs;
+	list_for_each_entry(fs, &fs_register_list, list) {
+		if (!strcmp(name, fs->name))
+			return 1;
+	}
+	return 0;
+}
+
 static int fs_register_add(char *name)
 {
 	struct fs_reg *fs;
+
+	if (fs_register_check(name))
+		return -EALREADY;
+
 	fs = malloc(sizeof(struct fs_reg));
 	if (!fs)
 		return -ENOMEM;
@@ -221,16 +236,6 @@ static void fs_register_del(char *name)
 	}
 }
 
-static int fs_register_check(char *name)
-{
-	struct fs_reg *fs;
-	list_for_each_entry(fs, &fs_register_list, list) {
-		if (!strcmp(name, fs->name))
-			return 1;
-	}
-	return 0;
-}
-
 #define MAXARGS 8
 
 static char *get_args(char *buf, int *argc, char **argv, char sep, int want)
@@ -465,9 +470,10 @@ static void query_dump_plocks(int fd, char *name)
 /* combines a header and the data and sends it back to the client in
    a single do_write() call */
 
-static void do_reply(int fd, int cmd, char *name, int result, char *buf,
-		     int buflen)
+static void do_reply(int fd, int cmd, char *name, int result, int option,
+		     char *buf, int buflen)
 {
+	struct dlmc_header *h;
 	char *reply;
 	int reply_len;
 
@@ -476,8 +482,10 @@ static void do_reply(int fd, int cmd, char *name, int result, char *buf,
 	if (!reply)
 		return;
 	memset(reply, 0, reply_len);
+	h = (struct dlmc_header *)reply;
 
-	init_header((struct dlmc_header *)reply, cmd, name, result, buflen);
+	init_header(h, cmd, name, result, buflen);
+	h->option = option;
 
 	if (buf && buflen)
 		memcpy(reply + sizeof(struct dlmc_header), buf, buflen);
@@ -507,7 +515,7 @@ static void query_lockspace_info(int fd, char *name)
 	else
 		rv = set_lockspace_info(ls, &lockspace);
  out:
-	do_reply(fd, DLMC_CMD_LOCKSPACE_INFO, name, rv,
+	do_reply(fd, DLMC_CMD_LOCKSPACE_INFO, name, rv, 0,
 		 (char *)&lockspace, sizeof(lockspace));
 }
 
@@ -528,7 +536,7 @@ static void query_node_info(int fd, char *name, int nodeid)
 	else
 		rv = set_node_info(ls, nodeid, &node);
  out:
-	do_reply(fd, DLMC_CMD_NODE_INFO, name, rv,
+	do_reply(fd, DLMC_CMD_NODE_INFO, name, rv, 0,
 		 (char *)&node, sizeof(node));
 }
 
@@ -556,7 +564,7 @@ static void query_lockspaces(int fd, int max)
 		result = ls_count;
 	}
  out:
-	do_reply(fd, DLMC_CMD_LOCKSPACES, NULL, result,
+	do_reply(fd, DLMC_CMD_LOCKSPACES, NULL, result, 0,
 		 (char *)lss, ls_count * sizeof(struct dlmc_lockspace));
 
 	if (lss)
@@ -599,7 +607,7 @@ static void query_lockspace_nodes(int fd, char *name, int option, int max)
 		result = node_count;
 	}
  out:
-	do_reply(fd, DLMC_CMD_LOCKSPACE_NODES, name, result,
+	do_reply(fd, DLMC_CMD_LOCKSPACE_NODES, name, result, 0,
 		 (char *)nodes, node_count * sizeof(struct dlmc_node));
 
 	if (nodes)
@@ -650,14 +658,12 @@ static void process_connection(int ci)
 		if (group_mode == GROUP_LIBGROUP) {
 			rv = -EINVAL;
 		} else {
-			rv = 0;
+			rv = fs_register_add(h.name);
 			ls = find_ls(h.name);
 			if (ls)
 				ls->fs_registered = 1;
-			else
-				rv = fs_register_add(h.name);
 		}
-		do_reply(client[ci].fd, DLMC_CMD_FS_REGISTER, h.name, rv,
+		do_reply(client[ci].fd, DLMC_CMD_FS_REGISTER, h.name, rv, 0,
 			 NULL, 0);
 		break;
 
@@ -676,8 +682,9 @@ static void process_connection(int ci)
 			rv = set_fs_notified(ls, h.data);
 		else
 			rv = -ENOENT;
+		/* pass back the nodeid provided by caller in option field */
 		do_reply(client[ci].fd, DLMC_CMD_FS_NOTIFIED, h.name, rv,
-			 NULL, 0);
+			 h.data, NULL, 0);
 		break;
 
 	case DLMC_CMD_DEADLOCK_CHECK:
@@ -834,6 +841,20 @@ static int setup_queries(void)
 	return 0;
 }
 
+/* The dlm in kernels before 2.6.28 do not have the monitor device.  We
+   keep this fd open as long as we're running.  If we exit/terminate while
+   lockspaces exist in the kernel, the kernel will detect a close on this
+   fd and stop the lockspaces. */
+
+static void setup_monitor(void)
+{
+	if (!monitor_minor)
+		return;
+
+	kernel_monitor_fd = open("/dev/misc/dlm-monitor", O_RDONLY);
+	log_debug("/dev/misc/dlm-monitor fd %d", kernel_monitor_fd);
+}
+
 void cluster_dead(int ci)
 {
 	log_error("cluster is down, exiting");
@@ -847,9 +868,6 @@ static void loop(void)
 	void (*workfn) (int ci);
 	void (*deadfn) (int ci);
 
-	/* FIXME: add code that looks for uncontrolled instances of
-	   dlm lockspaces in the kernel */
-
 	rv = setup_queries();
 	if (rv < 0)
 		goto out;
@@ -859,10 +877,10 @@ static void loop(void)
 		goto out;
 	client_add(rv, process_listener, NULL);
 
-	rv = setup_cman();
+	rv = setup_cluster();
 	if (rv < 0)
 		goto out;
-	client_add(rv, process_cman, cluster_dead);
+	client_add(rv, process_cluster, cluster_dead);
 
 	rv = setup_ccs();
 	if (rv < 0)
@@ -870,6 +888,16 @@ static void loop(void)
 
 	setup_logging();
 
+	rv = check_uncontrolled_lockspaces();
+	if (rv < 0)
+		goto out;
+
+	rv = setup_misc_devices();
+	if (rv < 0)
+		goto out;
+
+	setup_monitor();
+
 	rv = setup_configfs();
 	if (rv < 0)
 		goto out;
@@ -969,7 +997,7 @@ static void loop(void)
 	clear_configfs();
 	close_logging();
 	close_ccs();
-	close_cman();
+	close_cluster();
 
 	if (!list_empty(&lockspaces))
 		log_error("lockspaces abandoned");
@@ -1259,7 +1287,7 @@ int poll_ignore_plock;
 int plock_fd;
 int plock_ci;
 struct list_head lockspaces;
-int cman_quorate;
+int cluster_quorate;
 int our_nodeid;
 char daemon_debug_buf[256];
 char dump_buf[DLMC_DUMP_SIZE];
@@ -1268,4 +1296,46 @@ int dump_wrap;
 char plock_dump_buf[DLMC_DUMP_SIZE];
 int plock_dump_len;
 int group_mode;
+uint32_t control_minor;
+uint32_t monitor_minor;
+uint32_t plock_minor;
+uint32_t old_plock_minor;
+
+/* was a config value set on command line?, 0 or 1.
+   optk is a kernel option, optd is a daemon option */
+
+int optk_debug;
+int optk_timewarn;
+int optk_protocol;
+int optd_groupd_compat;
+int optd_debug_logsys;
+int optd_enable_fencing;
+int optd_enable_quorum;
+int optd_enable_deadlk;
+int optd_enable_plock;
+int optd_plock_debug;
+int optd_plock_rate_limit;
+int optd_plock_ownership;
+int optd_drop_resources_time;
+int optd_drop_resources_count;
+int optd_drop_resources_age;
+
+/* actual config value from command line, cluster.conf, or default.
+   cfgk is a kernel config value, cfgd is a daemon config value */
+
+int cfgk_debug                  = -1;
+int cfgk_timewarn               = -1;
+int cfgk_protocol               = -1;
+int cfgd_groupd_compat          = DEFAULT_GROUPD_COMPAT;
+int cfgd_debug_logsys           = DEFAULT_DEBUG_LOGSYS;
+int cfgd_enable_fencing         = DEFAULT_ENABLE_FENCING;
+int cfgd_enable_quorum          = DEFAULT_ENABLE_QUORUM;
+int cfgd_enable_deadlk          = DEFAULT_ENABLE_DEADLK;
+int cfgd_enable_plock           = DEFAULT_ENABLE_PLOCK;
+int cfgd_plock_debug            = DEFAULT_PLOCK_DEBUG;
+int cfgd_plock_rate_limit       = DEFAULT_PLOCK_RATE_LIMIT;
+int cfgd_plock_ownership        = DEFAULT_PLOCK_OWNERSHIP;
+int cfgd_drop_resources_time    = DEFAULT_DROP_RESOURCES_TIME;
+int cfgd_drop_resources_count   = DEFAULT_DROP_RESOURCES_COUNT;
+int cfgd_drop_resources_age     = DEFAULT_DROP_RESOURCES_AGE;
 
