@@ -33,8 +33,8 @@
 #include <dirent.h>
 #include <openais/saAis.h>
 #include <openais/saCkpt.h>
-#include <openais/cpg.h>
-#include <openais/service/logsys.h>
+#include <corosync/cpg.h>
+#include <corosync/engine/logsys.h>
 
 #include <linux/dlmconstants.h>
 #include "libgfscontrol.h"
@@ -48,7 +48,7 @@
 */
 
 /* Maximum members of a mountgroup, should match CPG_MEMBERS_MAX in
-   openais/cpg.h.  There are no max defines in gfs-kernel for
+   corosync/cpg.h.  There are no max defines in gfs-kernel for
    mountgroup members. (FIXME verify gfs-kernel/lock_dlm) */
 
 #define MAX_NODES       128
@@ -287,6 +287,7 @@ void cluster_dead(int ci);
 int setup_cman(void);
 void close_cman(void);
 void process_cman(int ci);
+void kick_node_from_cluster(int nodeid);
 
 /* plock.c */
 int setup_plocks(void);
@@ -310,6 +311,7 @@ int read_sysfs_int(struct mountgroup *mg, char *field, int *val_out);
 int run_dmsetup_suspend(struct mountgroup *mg, char *dev);
 void update_dmsetup_wait(void);
 void update_flow_control_status(void);
+int check_uncontrolled_filesystems(void);
 
 /* logging.c */
 
