@@ -80,10 +80,10 @@ generate_config_file()
 
 	generate_configTemplate "$generated_file" "$1"
 	echo "external_pid_file = '$PSQL_pid_file'" >> "$generated_file"
-	echo "listen_addresses = '$ip_comma'" >> "$generated_file"
+	echo "listen_address = '$ip_comma'" >> "$generated_file"
 
 	echo >> "$generated_file"	
-	sed 's/^[[:space:]]*external_pid_file/### external_pid_file/i;s/^[[:space:]]*listen_addresses/### listen_addresses/i' < "$original_file" >> "$generated_file"
+	sed 's/^[[:space:]]*external_pid_file/### external_pid_file/i;s/^[[:space:]]*listen_address/### listen_address/i' < "$original_file" >> "$generated_file"
 	
         sha1_addToFile "$generated_file"
 	clog_generate_config $CLOG_SUCCEED "$original_file" "$generated_file"
@@ -93,9 +93,6 @@ generate_config_file()
 
 start()
 {
-	declare ccs_fd;
-	declare pguser_group
-	
 	clog_service_start $CLOG_INIT
 
 	create_pid_directory
@@ -108,25 +105,10 @@ start()
 		return $OCF_ERR_GENERIC
 	fi
 
-	#
-	# Create an empty PID file for the postgres user and
-	# change it to be owned by the postgres user so that
-	# postmaster doesn't complain.
-	#
-	pguser_group=`groups $OCF_RESKEY_postmaster_user | cut -f1 -d ' '`
-	touch $PSQL_pid_file
-	chown $OCF_RESKEY_postmaster_user.$pguser_group $PSQL_pid_file
-
 	clog_looking_for $CLOG_INIT "IP Addresses"
 
-        ccs_fd=$(ccs_connect);
-        if [ $? -ne 0 ]; then
-		clog_looking_for $CLOG_FAILED_CCS
-                return $OCF_ERR_GENERIC
-        fi
-
-        get_service_ip_keys "$ccs_fd" "$OCF_RESKEY_service_name"
-        ip_addresses=`build_ip_list "$ccs_fd"`
+        get_service_ip_keys "$OCF_RESKEY_service_name"
+        ip_addresses=`build_ip_list`
 
 	if [ -z "$ip_addresses" ]; then
 		clog_looking_for $CLOG_FAILED_NOT_FOUND "IP Addresses"
@@ -137,8 +119,8 @@ start()
 
 	generate_config_file "$OCF_RESKEY_config_file" "$PSQL_gen_config_file" "$ip_addresses"
 
-	su - "$OCF_RESKEY_postmaster_user" -c "$PSQL_POSTMASTER -c config_file=\"$PSQL_gen_config_file\" \
-		$OCF_RESKEY_postmaster_options" &> /dev/null &
+	sudo -u "$OCF_RESKEY_postmaster_user" $PSQL_POSTMASTER -c config_file="$PSQL_gen_config_file" \
+		$OCF_RESKEY_postmaster_options 
 
 	if [ $? -ne 0 ]; then
 		clog_service_start $CLOG_FAILED
